/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozCC
 *
 * The Initial Developer of the Original Code is
 * Nathan R. Yergler.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Nathan R. Yergler <nathan@yergler.net>
 *
 * ***** END LICENSE BLOCK ***** 
 *
 * $Id: license.js,v 1.2 2003/12/09 14:27:22 nathan Exp $
 *
 */

include ('chrome://mozcc/content/mozcc.js');
include ('chrome://mozcc/content/rdfParser.js');

var g_attrsSeen = new Array();

// parseRdf
/*  Takes a list of strings, each containing an RDF block;
 *  returns a list of RDF() objects.
 */
function parseRdf(segments) {

   var parsed = new Array();
   var parser;

   for (idx=0; idx < segments.length; idx++) {
        parser = new jlRDF();
	parser.parseRDF(segments[idx], noop, [])

        parsed.push(parser);

   } // for each segment

   for (foo=0; foo < parsed.length; foo++) {
     alert(parsed[foo].getTriples());
   }

   return parsed;

   function noop(args) {
      return true;
   }
      
} // parseRdf

// parent and child are both treeitems
function appendChild(parent, child) {

   var treechildren = parent.getElementsByTagName('treechildren');

   if (treechildren.length == 0) {
      // add the treechildren element
      children = document.createElement("treechildren");
      parent.appendChild(children);
   } else {
     children = treechildren[0];
   }

   children.appendChild(child);
}

function createChild(contents, container) {

   var child = document.createElement("treeitem");
   child.setAttribute("container", container);
   child.setAttribute("open", true);


   var row = document.createElement("treerow");
   
   for (i=0; i < contents.length; i++) {
      var cell = document.createElement("treecell");
      cell.setAttribute("label", contents[i]);

      // check if this cell is a link
      if ((/http:/gi).test(contents[i])) {
         cell.setAttribute("properties", "isLink");
         cell.setAttribute("tooltiptext", "double click to open URI");
      }

      row.appendChild(cell);
   }

   child.appendChild(row);

   return child;
}

function rpart(instr) {
   return instr.split('/').reverse()[0]
}

function isSubObject(object) {
  return (object.indexOf('genid:') > -1);
}

function isType(predicate) {
  return (predicate.indexOf('#') > -1);
}

function beenSeen(item) {
   // checks for membership in g_attrsSeen

   for (i=0; i < g_attrsSeen.length; i++) {
      if (g_attrsSeen[i] == item) return true;
   }

   return false;

}

function setSeen(item) {
   // mark an attr as seen
   g_attrsSeen.push(item);
}

function showAttrs(rdfParser, attrs, parent) {

   for (a = 0; a < attrs.length; a++) {
      // check if this is a type declaration
      if (isType(attrs[a].predicate)) continue;

      // check if this is a subobject
      if (isSubObject(attrs[a].object)) {
       if (!(beenSeen(attrs[a].object))) {
         setSeen(attrs[a].object);

         attr = createChild([rpart(attrs[a].predicate)], true);
         appendChild(parent, attr);

         sub_attrs = rdfParser.Match(null, attrs[a].object, null, null);

         showAttrs(rdfParser, sub_attrs, attr);
       }
      } else {
         attr = createChild([rpart(attrs[a].predicate), 
                             attrs[a].object], false);
         appendChild(parent, attr);
      }
   } // for each attribute

}

function onLoadLicense() {

   status_msg = window.document.getElementById("rdf_status");
   rdf_url    = window.document.getElementById("rdf_url");
   rdf_tree   = window.document.getElementById("rdf_info");
   rdf_children = window.document.getElementById("rdf_children");

   rdfStrings = extractRdf(window.arguments[0]); //.documentElement.innerHTML);

   // update the status message
   status_msg.value = "Parsed " + rdfStrings.length + " RDF segments from";
   rdf_url.value = window.arguments[0].URL;

   // for each segment, add a node
   for (s = 0; s < rdfStrings.length; s++) {
      var parsedRdf = new jlRDF();
      parsedRdf.parseRDF (rdfStrings[s], noop, []);

      segment = createChild(['Segment ' + (s+1)], true);
      rdf_children.appendChild(segment);

      // for work information, add a node
      works = parsedRdf.Match(null, null, 
              'http://www.w3.org/1999/02/22-rdf-syntax-ns#type', 
              'http://web.resource.org/cc/Work');
      for (w = 0; w < works.length; w++) {
         work = createChild([works[w].subject], true);
         appendChild(segment, work);

         // add a node for each work attribute
         attrs = parsedRdf.Match(null, works[w].subject, null, null);
         g_attrsSeen = new Array();
         showAttrs(parsedRdf, attrs, work);

      }  // for each work

      // for each license add a node
      licenses = parsedRdf.Match(null, null,
                 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type', 
                 'http://web.resource.org/cc/License');
      for (l=0; l < licenses.length; l++) {
         license = createChild([licenses[l].subject], true);
	 appendChild(segment, license);

         // add a node for each attribute
         attrs = parsedRdf.Match(null, licenses[l].subject, null, null);
         g_attrsSeen = new Array();
         showAttrs(parsedRdf, attrs, license);

      } // for each license
   } // for each segment

  return true;
}

/* dblClickRow
 *   Fired when user double-clicks a row in the tree.  Determines
 *   which column was clicked, checks if the cell is a link and if
 *   so opens the URI in a new window.
 */
function dblClickRow(event) {

   rdf_tree     = window.document.getElementById("rdf_info");
   rdf_children = window.document.getElementById("rdf_children");

   pred_col = window.document.getElementById("predicate");

   // determine which column was clicked
   if (event.pageX <= pred_col.boxObject.width) {
      celltext = rdf_tree.view.getCellText(rdf_tree.currentIndex, "predicate");
   } else {
      celltext = rdf_tree.view.getCellText(rdf_tree.currentIndex, "object");
   }

   // check if the clicked area is a link
   if ((/http:/gi).test(celltext)) {
      openUrl(celltext);
   }
   
}


function noop(args) {
   return true;
}

/* openUrl(startpage)
 *   Opens the given url (startpage) in a new browser window.
 *   Adapted from browser.js; copyright, license and source at
 *  http://lxr.mozilla.org/mozilla/source/browser/base/content/browser.js
 */
function openUrl(startpage) {

   var charsetArg = new String();
   var handler = Components.classes['@mozilla.org/commandlinehandler/general-startup;1?type=browser'];
   handler = handler.getService();
   handler = handler.QueryInterface(Components.interfaces.nsICmdLineHandler);

   var url = handler.chromeUrlForTask;
   var wintype = document.firstChild.getAttribute('windowtype');
 
   // if and only if the current window is a browser window and 
   // it has a document with a character
   // set, then extract the current charset menu setting from 
   // the current document and use it to
   // initialize the new browser window...
   if (window && (wintype == "navigator:browser") && 
       window._content && window._content.document)
   {
     var DocCharset = window._content.document.characterSet;
     charsetArg = "charset="+DocCharset;
 
     //we should "inherit" the charset menu setting in a new window
     window.openDialog(url, "_blank", "chrome,all,dialog=no", 
                       startpage, charsetArg);
   }
   else // forget about the charset information.
   {
     window.openDialog(url, "_blank", "chrome,all,dialog=no", startpage);
   }

}

